;; The Tower represented by a list of numbers
 
;; Create empty tower
(defun make-empty-tower () nil)
 
;; Create tower by stacking DISK on top of TOWER
(defun tower-push (tower disk) (cons disk tower))
 
;; Get the top disk of TOWER
(defun tower-top (tower) (first tower))
 
;; Remove the top disk of TOWER
(defun tower-pop (tower) (rest tower))

;; Create a Hanoi Configuration
(defun make-hanoi (from-tower aux-tower to-tower)
  (list from-tower aux-tower to-tower))
 
;; Select I'th tower of HANOI
(defun hanoi-tower (hanoi i)
  (nth (1- i) hanoi))

;; replace the I'th tower in the HANOI configuration by tower TOWER
(defun hanoi-tower-update (hanoi i tower)
  (cond
    ((= i 1) (make-hanoi tower (second hanoi) (third hanoi)))
   ((= i 2) (make-hanoi (first hanoi) tower (third hanoi)))
   ((= i 3) (make-hanoi (first hanoi) (second hanoi) tower))))
 
;; return the top disk from I'th tower of HANOI
(defun hanoi-tower-top (hanoi i)
  (tower-top (hanoi-tower hanoi i)))
 
;; Pop the top disk of the I'th tower in the HANOI configuration
(defun hanoi-tower-pop (hanoi i)
  (hanoi-tower-update hanoi i (tower-pop (hanoi-tower hanoi i))))
 
;; Push DISK into I'th tower of HANOI
(defun hanoi-tower-push (hanoi i disk)
  (hanoi-tower-update hanoi i (tower-push (hanoi-tower hanoi i) disk)))

(defun move-disk (from to hanoi)
  (let
    ((disk (hanoi-tower-top hanoi from))
      (intermediate-hanoi (hanoi-tower-pop hanoi from)))
  (hanoi-tower-push intermediate-hanoi to disk)))

;; N disks from peg FROM to peg TO using AUX peg as in HANOI
(defun move-tower (N from aux to hanoi)
  (if (= N 1)
    (move-disk from to hanoi)
      (move-tower (- N 1) aux from to 
        (move-disk from to
          (move-tower (- N 1) from to aux hanoi)))))

(defun solve-hanoi (N)
  (move-tower N 1 2 3 (make-hanoi (make-complete-tower N) nil nil)))
 
;; Create tower of n disks
(defun make-complete-tower (N)
  (make-complete-tower-aux N (make-empty-tower)))
 
;; Push a complete tower of N disks on top of A tower
(defun make-complete-tower-aux (N A)
  (if (zerop N)
    A
    (make-complete-tower-aux (1- N) (tower-push A N))))


